/*
 *  @file   ClockOps.c
 *
 *  @brief      Generic clock module interface across platforms and OS
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */


/* Standard headers */
#include <ti/syslink/Std.h>
/* Module level headers */
#include <ti/syslink/inc/ClockOps.h>
/* OSAL & Utils headers */
#include <ti/syslink/utils/Trace.h>

#if defined (__cplusplus)
extern "C" {
#endif


/* =============================================================================
 * APIs
 * =============================================================================
 */
/*!
 *  @brief      Function returns the Clock handle
 *
 *  @handle     Ptr to the clockOps object.conaining the clock function.table
 *  @clkname    name of clk for which handle is to be obtained
 *
 *  @sa         ClockOps_put
 */
Ptr
ClockOps_get (ClockOps_Handle handle, String clkName)
{

    return(handle->clkFxnTable.get(clkName));
}

/*!
 *  @brief       Function to release a Clock
 *
 *  @handle      Ptr to the clockOps object.conaining the clock function.table
 *  @clkHandle   Handle to the clock
 *
 *  @sa         ClockOps_get
 */
Void
ClockOps_put(ClockOps_Handle handle, Ptr clkHandle)
{
    handle->clkFxnTable.put(clkHandle);
}
/*!
 *  @brief       Function to enable a Clock
 *
 *  @handle      Ptr to the clockOps object.conaining the clock function.table
 *  @clkHandle   Handle to the clock
 *
 *  @sa         ClockOps_disable
 */
Int32
ClockOps_enable(ClockOps_Handle handle, Ptr clkHandle)
{
    return (handle->clkFxnTable.enable(clkHandle));
}

/*!
 *  @brief       Function to disable a Clock
 *
 *  @handle      Ptr to the clockOps object.conaining the clock function.table
 *  @clkHandle   Handle to the clock
 *
 *  @sa         ClockOps_enable
 */
Void
ClockOps_disable(ClockOps_Handle handle, Ptr clkHandle)
{
    handle->clkFxnTable.disable(clkHandle);
}

/*!
 *  @brief      Function gets the Clock speed .
 *
 *  @handle     Ptr to the clockOps object.conaining the clock function.table
 *  @clkHandle   Handle to the clock
 *
 *  @sa         ClockOps_setRate
 */
ULong
ClockOps_getRate(ClockOps_Handle handle, Ptr clkHandle)
{
    return(handle->clkFxnTable.getRate(clkHandle));
}

/*!
 *  @brief       Function sets the dlock speed
 *
 *  @handle      Ptr to the clockOps object.conaining the clock function.table
 *  @clkHandle   Handle to the clock
 *  @rate        Clock speed to be set
 *
 *  @sa         ClockOps_getRate
 */
Int32
ClockOps_setRate(ClockOps_Handle handle, Ptr clkHandle, ULong rate)
{
    return (handle->clkFxnTable.setRate(clkHandle, rate));
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
