/** 
 *  @file   IFrameQ.xdc
 *
 *  @brief      Interface file for FrameQ.
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */






package ti.syslink.ipc.rtos.interfaces;

import xdc.runtime.IGateProvider;
import ti.sdo.utils.NameServer ;
import ti.sdo.utils.MultiProc ;
import ti.syslink.ipc.rtos.interfaces.SysLink ;

interface IFrameQ  inherits IFrame {

   /*! @_nodoc
     *  Delegate writer needs...
     */
    const UInt32 CREATED      = 0x05251995;
    const UInt32 VERSION      = 1;

    /*! Type of creation */
    const UInt32 STATIC_CREATE                 = 0x1;
    /*! Type of creation */
    const UInt32 STATIC_CREATE_USEDREGION      = 0x2;
    /*! Type of creation */
    const UInt32 DYNAMIC_CREATE                = 0x4;
    /*! Type of creation */
    const UInt32 DYNAMIC_CREATE_USEDREGION     = 0x8;
    /*! Type of open */
    const UInt32 STATIC_OPEN                   = 0x10;
    /*! Type of open */
    const UInt32 DYNAMIC_OPEN                  = 0x20;
    /*! Type of open */
    const UInt32 INVALID                       = 0xFF;

    /*!
     *  Maximum length of the  name strings .
     */
    const UInt32  MAX_NAMELEN = 32 ;

    /*!
     *============CACHE_LINESIZE ================
     * Cache line size.
     *
     */
    const  UInt32 CACHE_LINESIZE = 128 ;

    /*!
     *  Maximum number of instances managed by FrameQ module.
     */
    const UInt32 MAX_INSTANCES = 64 ;

    /*!
     *  Maximum number of reader clients supported by an instance
     */
     const UInt32 MAX_INST_READERS = 2 ;

   /*!
     *  Notify Event Number to be used by  FrameQ module.
     */
    const UInt16 NOTIFY_RESERVED_EVENTNO  =  1 ;

    /*!
     *============maxFilledQueusForReader ================
     * Max filled queues  for a reader client of the FrameQ instance..
     *
     */
    const  UInt32 MAXFILLEDQUEUS_FOR_READER = 16 ;

    /*!
     *  If set  to true FrameQBufMgr  instances will use dafault gate set in
     *  FrameQBufMgr.
     *  If default gate is not set in FrameQBufMgr, it uses GateMP's default
     *  gate. Set it to zero if   FrameQBufMgr needs to  create  gate for a
     *  instance (remoteProtect and localProtect  flags should be valid in
     *  params in this case).
     */
    config Bool usedefaultgate   = false;
    /*!
     *  ======== OpenMode ========
     *  Enumeration of FrameQ open modes.
     *
     *  @value(MODE_WRITER) if specifed in openParams, open  call returns a writer handle.
     *
     *  @value(MODE_READER) if specifed in params, open call returns a reader handle.
     *
     */
    enum OpenMode {
        MODE_NONE,
        MODE_WRITER =1,
        MODE_READER =2
    }

    /*!
     *  ======== Config ========
     *  module config structure.
     *
     *  @field(eventNo)        Notify event number used for notification.
     */
    struct Config
    {
        UInt32 eventNo;
        UInt8  maxInstances;
    }

    /*!
     *  ======== Interface ========
     *  Enumeration of FrameQ interface types.Denotes different implementation types.
     *
     *  @value(INTERFACE_SHAREDMEM) FrameQ implementation (FrameQ_ShMem Implementation) on shared memory.
     *
     */

    enum Interface{
        INTERFACE_SHAREDMEM = 0x0,
        INTERFACE_NONE = 0x1
    }

    /*!
     *  ======== CpuAccessFlags ========
     *  Enumeration of CPU access flags for frame buffers. For frame headers CPUACCESS is assumed to be TRUE.
     *
     *  @value(FRAMEBUF0_CPUACCESS) If specified first frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF1_CPUACCESS) If specified second frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF2_CPUACCESS) If specified third frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF3_CPUACCESS) If specified fourth frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF4_CPUACCESS) If specified fifth frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF5_CPUACCESS) If specified sixth frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF6_CPUACCESS) If specified seventh frame buffer is accessed  through CPU( direct memory access).
     *  @value(FRAMEBUF7_CPUACCESS) If specified eigth frame buffer is accessed  through CPU( direct memory access).
     *
     */
    enum CpuAccessFlags{
        FRAMEBUF0_CPUACCESS    =  0x10000,
        FRAMEBUF1_CPUACCESS    =  0x20000,
        FRAMEBUF2_CPUACCESS    =  0x40000,
        FRAMEBUF3_CPUACCESS    =  0x80000,
        FRAMEBUF4_CPUACCESS    = 0x100000,
        FRAMEBUF5_CPUACCESS    = 0x200000,
        FRAMEBUF6_CPUACCESS    = 0x400000,
        FRAMEBUF7_CPUACCESS    = 0x800000
    }
   const UInt32 FRAMEBUF_CPUACCESSFLAGS_BITOFFSET = 16;

    /*!
     *  ========= CreateParams ==========
     *  Structure defining common instance parameters for the FrameQ
     *  module.Should be the first element  in the create params strcture of
     *  the actual implementation.
     *  @field(size)  Size of the Create params structure(Should be the size of the actual
     *                implementation's Create params structure in which this FrameQ_CreateParams is the
     *                first element.
     *  @field(interface) type of implementation. See {@link Interface}.
     *  @field(name)  Name of the instance.
     *  @field(openFlag) Denotes whether create should actually create/open instance.
     */
    struct CreateParams {
        UInt32            size;
        Interface         ctrlInterfaceType;
        String            name;
        Bool              openFlag;
    } ;

    /*!
     *  ========= CommonOpenParams ==========
     *  Structure defining common instance  open parameters for the FrameQ
     *  module.Should be first element  in the open params strcture of the
     *  actual implementation.
     *
     *  @field(name)      Name of the instance.
     *  @field(sharedAddr)virtual address of the instance shared
         *                    control structure.
     *  @field(openMode)  Mode of the client
     *  @field(cpuAccessFlags) Flag to specify if  Frame buffers   are  accessed
     *                    through  CPU.see {@link CpuAccessFlags}.
     */
     struct CommonOpenParams{
        String      name;
        Ptr         sharedAddr;
        UInt32      openMode;
        UInt32      cpuAccessFlags;
    };

  /*!
   *  @brief   Enumerates the notification types used by SysLink ipc components.
   */
    /*!
     *  ======== NotifyType ========
     *  Enumeration of Client notifyMgr notification types..
     *
     *  @value(NOTIFICATION_NONE)   No notification required.
     *
     *  @value(NOTIFICATION_ALWAYS) Notify whenever the other client sends data/frees up space.
     *
     *  @value(NOTIFICATION_ONCE)   Notify when the other side sends data/frees up buffer. Once the
     *                              notification is done, the notification is disabled until it is
     *                              enabled again.
     *
     *  @value(NOTIFICATION_HDWRFIFO_ALWAYS) Notify whenever the other side sends data/frees up space.
     *                                       This notification is never disabled.
     *
     *  @value(NOTIFICATION_HDWRFIFO_ONCE)   Notify when the other side sends data/frees up buffer. Once the
     *                                       notification is done, the notification is disabled until it is
     *                                       enabled again. The notification is enabled once the watermark
     *                                       is crossed and does not require buffer to get full/empty..
     */
    enum NotifyType {
                NOTIFICATION_NONE   = 0,
                NOTIFICATION_ALWAYS = 1,
                NOTIFICATION_ONCE   = 2,
                NOTIFICATION_HDWRFIFO_ALWAYS = 3,
                NOTIFICATION_HDWRFIFO_ONCE   = 4
        } ;
    /*!
     *  ========  NotifyFunc ========
     *  Prototype of the FrameQ call back function.
     *
     *
     */
     typedef void (* NotifyFunc) ( Handle , Ptr , UInt16) ;

    /*!
     *  ========= NotifyParams ==========
     * Client notification registration parameter structure.
     *
     *  @field(notifyType) notification type.See {@link SysLink.NotifyType}.
     *  @field(watermark) Minumum number of free frames required to generate
     *                    notification   Same water mark for all the individual
     *                    freeFramePools in the set.
     *  @field(cbFxnPtr)  Cacll back function.
     *  @field(cbContext) Context pointer that needs to be passed to call back
     *                    function.
     *  @field(cbClientHandle) Handle that needs to be passed as first arg to
     *                         call back.Ideally this should be callers client
     *                         handle.
     *
     */
     struct NotifyParams {
//         SysLink.NotifyType           notifyType;
         NotifyType                     notifyType;
         UInt32                         watermark;
         NotifyFunc                     cbFxnPtr;
         Ptr                            cbContext;
         Ptr                            cbClientHandle;
    };

   /*!
     *  ======== FrameQueue ========
     *  Structure to maintain the queue related information.
     *
     *  @field(readerIndex) Reader identifier.
     *  @field(numFrames)  Number of frames available in the queue.
     *  @field(isAllocated)    Flag  denoting whether queue is allocated to any reader or not.
     */
    struct FrameQueue {
        volatile UInt32          readerIndex ; /* To identify the reader   to which this queue is allocated.  */
        volatile UInt32          numFrames[MAXFILLEDQUEUS_FOR_READER] ; /* Number of nodes in  filled queues of the reader */
        volatile Bool            isAllocated ;
    }

   /*!
     *  ======== Attrs ========
     *  Shared  control strucrue for the instance attributes.
     *
     *  @field(status)         flag  denoting whether instance is created or not.
     *  @field(version)        Version of the  instance.
     *  @field(localProtect)   Local protection type.See {@link #GateMP.LocalProtect}
     *  @field(remoteProtect)  Remote protection types.See {@link #GateMP.RemoteProtect}
     *  @field(gateMPAddr)     GateMP address (shm safe)
     *  @field(numReaders)     Max reader clients supported for this instance.
     *  @field(numActiveReaders) Number of readers currenty opened for the instance.
     *  @field(numQueues)        Number of queues cofigured for each reader.
     *  @field(primaryQueueId)   QueueId to which writer sends frames if there are no active readers  or only one reader exist.
     *  @field(frmQBufMgrId)     Id of FrameQBufmgr used by this instance.
     *  @field(frmQueue)         Array of frame queues supported for this instance.
     */

    struct Attrs {
        volatile UInt32      status;
        volatile UInt32      version;
        volatile UInt32      localProtect;
        volatile UInt32      remoteProtect;
        volatile Ptr         gateMPAddr; /* GateMP address (shm safe)     */
        volatile UInt32      numReaders; /* Number of reader clients currently using this instance */
        volatile UInt32      numActiveReaders ;/* Number of readers who are currently active.*/
        volatile UInt32      numQueues;
        volatile UInt32      primaryQueueId ; /* Queue to which writer sends frames if there are no active readers exist or one reader exist*/
        volatile UInt32      frmQBufMgrId;
    }

   /*!
     *  ======== WriterClient ========
     *  Writer client info  in shared memory.
     *
     *  @field(isValid)   Denotes whether client is valid  or not.
     *  @field(cacheFlags) Cache flags for the frame buffer and frame header.  Shared structure cache flag is specified in the object.
     *
     */
    struct WriterClient {
        volatile UInt32      isValid ;
        volatile UInt32      procId;
        volatile UInt32      cacheFlags;
        volatile Bool        isRegistered;
    }

    /*!
     *  ======== ReaderClient ========
     *  Reader client info structure.
     *
     *  @field(isValid)   Denotes whether client is valid  or not.
     *  @field(cacheFlags) Cache flags for the frame buffer and frame header.
     *  @field(frameQIndex) frameQueue from which  this reader can read the frames.
     *
     */
    struct ReaderClient {
        volatile UInt32      isValid ;
        volatile UInt32      procId;
        volatile UInt32      cacheFlags ;
        volatile UInt32      frameQIndex ;
        volatile UInt32      notifyId ;
        volatile Bool        isRegistered ;
    }

    /*! Initialize the passed pointer with the current parameters
     *
     *  @param(cfgParams)   Pointer to parameter structure to be initialized.
     */
    Int32 getConfig (Config * cfgParams) ;


    /*!
     *  Whether to have this module use the NameServer or not.
     *
     *  Currently defaulted to false because static creation of NameServer
     *  instances is not supported.
     */
    metaonly config Bool useNameServer = true;
    /*!
     *  ======== nameSrvPrms ========
     *  This Params object is used for temporary storage of the
     *  module wide parameters that are for setting the NameServer instance.
     */
    metaonly config NameServer.Params nameSrvPrms;

    /*!
     *  Section name is used to place the names table
     */
    metaonly config String tableSection = null;


   /*!
     *============nameServer ================
     * Handle to the name server.
     *
     */
     config NameServer.Handle  nameServer = null;

    /*!
     *  ======== setFrameBufValidSize ========
     *  Function to set valid data size of a frame buffer .
     *
     *  @param(buf) frame.See {@link Frame}.
     *  @param(frameBufNum) Number of the framebuffer to which this setting is
     *                      required.
     *  @param(validSize)   valid data size value to set in for the frame buffer.
     *  @b(returns) Returns valid data size in the frame buffer.
     *
     */
     Int32 setFrameBufValidSize (Frame frame,
                                UInt32 frameBufNum,
                                UInt32 validSize) ;

    /*!
     *  ======== setFrameBufDataStart ========
     *  Function to set data start offset from where valid data starts
     *              in frame buffer .
     *
     *  @param(frame)  frame.See {@link Frame}
     *  @param(frameBufNum) Number of the framebuffer to which this setting is required.
     *  @param(dataStartOffset) dataStart offset value.
     *
     *  @b(returns) Returns  data start offset from where valid data starts
     *              in frame buffer.
     *
     */
     Int32 setFrameBufDataStartOffset (Frame frame, UInt32 frameBufNum,
                                        UInt32 dataStartOffset) ;

    /*!
     *  ======== getNumFrameBuffers ========
     *  Function to know the number of frame buffers contained in a frame.
     *
     *  @param(frame) frame pointer(i.e frame header).See {@link Frame}
     *
     *  @b(returns) Returns  number of frame buffers in given frame.
     *
     */
     UInt32  getNumFrameBuffers (Frame frame ) ;

    /*!
     *  ======== getFrameBuffer ========
     *  Function to get the framebuffer.If frame is extended it returns the pointer
     *              to the user supplied buffer .
     *
     *  @param(frame) Pointer to the frame. See {@link Frame}
     *  @param(frameBufNum) Number of the framebuffer  that needs to returned.see {@link #getNumFrameBuffers}
     *
     *  @b(returns) Returns  framebuffer pointed to by the given number in a frame .
     *
     */
     Ptr  getFrameBuffer (Frame frame , UInt32 frameBufNum) ;


    /*!
     *  ======== getFrameBufValidSize ========
     *  Function to get valid data size of  a  frame buffer.
     *
     *  @param(frame)        Frame.See {@link Frame}
     *  @param(frameBufNum)  Frame Buffer number.
     *
     *  @b(returns) Returns  valid data size of data in the given frame.
     *
     */
     UInt32 getFrameBufValidSize (Frame frame, UInt32 frameBufNum ) ;

    /*!
     *  ======== getFrameBufDataStart ========
     *  Function to get data start offset in frame buffer .
     *
     *  @param(frame)        frame.See {@link Frame}.
     *  @param(frameBufNum)  Frame Buffer number.
     *
     *  @b(returns) Returns  valid data start offset in frame buffer.
     *
     */
     UInt32 getFrameBufDataStartOffset (Frame frame, UInt32 frameBufNum) ;

     /*!
     *  ======== getFrameBufSize ========
     *  Function to get size of frame buffer.
     *
     *  @param(frame)        frame.See {@link Frame}.
     *  @param(frameBufNum)  Frame Buffer number.
     *
     *  @b(returns) Returns  frame buffer size of the given frame number.
     *
     */
     UInt32 getFrameBufSize (Frame frame, UInt32 frameBufNum) ;





instance:

    // config CreateParams   commonParams;

    /*!
     *  ======== alloc ========
     *  Function to allocate a frame .
     *
     * It allocates frame from the free Queue 0 of FrameQBufMgr
     *  @param(framePtr)       Location to receive the allocated frame.
	 *
     *  @b(returns)       returns 0 if  successfully allocated the buffer.
     *
     */
    Int32 alloc (Frame       * framePtr);

    /*!
     *  ======== allocv ========
     *  Function to allocate  multiple frames from the FrameQ. FrameQ calls
	 *  alloctes frames from the plugged in  FrameQBufMgr instance from  the
	 *  give free Frame pool nos .
     *
     *  @param(framePtr)  array of pointers to receive allocate frame pointers.
     *
     *  @param(freeQId)   array of freeFrame poolNos to allocate frames from.
     *                    if API retruns success, this represents ths actual size of the buffer allocated.
     *  @param(numFrames)      Number of frames to allocate.
     *
     *  @b(returns)       returns value greater than zero
	 *                    if  successfully allocated frames.
     *
     */
	Int32 allocv (    Frame                framePtr[],
                      UInt32               freeQId[],
                      UInt8               *numFrames);
    /*!
     *  ======== free  ========
     *  Function to free frame.
     *
     *  @param(frame)       Frame to be freed.
     *
     *
     *
     *  @b(returns)       returns 0 if  successfully freed the  allocated buffer.
     *
     */
    Int32 free (Frame  frame );

    /*!
     *  ======== freev  ========
     *  Function to free frames
     *
     *  @param(framePtr)       array of frames to be freed.
     *
	 *  @b(returns)       returns 0 if  successfully freed the  allocated frames.
     *
     */
	 Int32	freev (Frame  framePtr[], UInt32 numFrames);

	/*!
     *  ======== put ========
     *  Function to insert frames in to FrameQ instance.
     *
     *
     */
    Int32 put  (Frame  buf ) ;

	/*!
	 * ======== putv ========
	 * Function to insert multiple frames  in to queues of a
	 * FrameQ reader client.
	 */
    Int32 putv (Frame framePtr[], UInt32 filledQueueId[], UInt8 numFrames);

    /*!
     *  ======== get ========
     *  Function to retrieve(get) frames from FrameQ.
     *
     *
     */
    Int32 get  (Frame* frame) ;

	/*
	 * ======== getv ========
	 * Function to retrieve frames from  queues of a FrameQ Reader client.
	 * It returns frame if it is available in the queue.
	 */
	Int32 getv (Frame pframe[], UInt32 filledQueueId[], UInt8 *numFrames);

    /*!
     *  ======== dup ========
     *  Function to duplicate frame.
     *
     *
     */
    Int32 dup  (Frame frame, Frame *dupedFrame);

    /*!
     *  ======== registerNotifier ========
     *  Function to register a call back function with the instance..
     *
     *  @param(notifyParams)  Notification registration parameters.See {@link NotifyRegParams}.
     *
     *  @b(returns)                 Returns 0 if  successfully registered the call back function.
     *
     */
    Int32 registerNotifier (NotifyParams *notifyParams);

    /*!
     *  ======== unregisterNotifier ========
     *  Function to unregister the call back function .
     *
     *
     *  @b(returns)                 Returns positive value  if  successfully unregistered the call back function.
     *
     */
    Int32 unregisterNotifier ();
    /*!
     *  ======== sendNotify ========
     *  Function to send force notification to reader client .
     *
     *  @b(returns)   Returns positive value if successfully sent notification.
     *
     */
	Int32 sendNotify (UInt16 msg);
    /* FrameQ helper API */

   /*!
     *  ======== getNumFrames ========
     *  Function to get the number of frames .
     *
     *  @param(numFrames) Outputparameter indicates number of frames in the FrameQ.
     *                    For readers, this will  indicates the number of frames present in the queue that
     *                                 it is accessing.
     *                    For writer, this indicates the number frames present in the primary queue.
     *
     *  @b(returns)       returns 0 if  successfully returned the frames.
     *
     */
     Int32 getNumFrames (UInt32* numFrames) ;

    /*!
     *  ======== getvNumFrames ========
     * For Reader this function is to get the available frames  from the queues
     * associated with this reader client.
     * For writer this function is to get the available frames  from the queues
     * associated with the primary Reader(first reader).
	 *
     *  @param(numFrames) Outputparameter indicates number of frames in each 
	 *                    given filled quque.
	 *  @param(filledQId) Arry of filled queue ids.
	 *  @param(numFilledQids) Number of filled queueids .
     *
     *  @b(returns)       returns 0 if  successfully returned the frames.
     *
     */
	Int32 getvNumFrames(UInt32            numFrames[],
                        UInt8             filledQId[],
                        UInt8             numFilledQids);

   /*!
     *  ======== getNumFreeFrames ========
     *  Function to get the number of free frames available in the plugged in 
	 *  FrameQbufMgr's FreeFramePool 0. The returned number free frame may not 
	 *  exist in the frameQbufMgr  after this call.This is because 
	 *  if any allocs are done by the other FrameQ writers on the 
	 *  same FrameQBufMgr.If frees are not other frameQ writers the free Frames 
	 *  would be even more than the value that we have got.
     *
     *  @param(numFreeFrames) Outputparameter indicates number of free frames 
	 *                    .
     *
     *  @b(returns)       returns 0 if  successfully returned the number of 
	 *                    free frames.
     *
     */
     Int32 getNumFreeFrames (UInt32* numFreeFrames) ;

    /*!
     *  ======== getvNumFreeFrames ========
     * Function to get the number of free frames available in the plugged in 
	 * FrameQbufMgr. 
	 *
     *  @param(numFreeFrames) Outputparameter indicates number of free frames in
	 *                    each given free quque at that instance.
	 *  @param(filledQId) Array of free Queue ids( freeFramePool nos).
	 *  @param(numFreeQids) Number of free queueids same as freeFrame PoolNo of 
	 *                      plugged in FrameQBufMgr .
     *
     *  @b(returns)       returns 0 if  successfully returned the frames.
     *
     */
	Int32 getvNumFreeFrames(UInt32            numFreeFrames[],
                            UInt8             freeQId[],
                            UInt8             numFreeQids);

/*!
 *  ======== FrameQ_control ========
 *  Provides a hook to perform implementation dependent operation.
 *
 *  @param(cmd) command to exectue.
 *  @param(arg) argument of type void*.
 */
	Int32 control (Int32 cmd, Ptr arg);
}

