/** 
 *  @file   FrameQ_ShMem.xdc
 *
 *  @brief      XDC file
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */


package ti.syslink.ipc.rtos.shMem


import xdc.runtime.Error;
import ti.sdo.ipc.ListMP ;
import ti.sdo.utils.NameServer;
import xdc.runtime.IGateProvider;
import ti.sdo.ipc.GateMP ;
import ti.syslink.ipc.rtos.FrameQBufMgr;

@ModuleStartup
@InstanceInitError /* initialization mat throw errors */
@InstanceFinalize
module FrameQ_ShMem inherits ti.syslink.ipc.rtos.interfaces.IFrameQ {

    /*!
     *  Assert raised when an invalid parameter(s) is passed into a function
     */
    config xdc.runtime.Assert.Id A_invalidParams =
        {msg: "A_invalidParams: Invalid parameter"};

    /*!
     *  Assert raised when a null pointer needs to be dereferenced
     */
    config xdc.runtime.Assert.Id A_nullPointer =
        {msg: "A_nullPointer: Null pointer"};

    /*!
     *  Assert raised when an API call failed.
     */
    config xdc.runtime.Assert.Id A_apiFail =
        {msg: "A_apiFail:  api call failed"};

    /*!
     *  Assert raised when n invalid shared region address  specified.
     */
    config xdc.runtime.Assert.Id A_InvalidSharedRegionAddr =
        {msg: "A_InvalidSharedRegionAddr: Invalid shared region address"};

    /*!
     *  Assert raised when n invalid shared region address  specified.
     */
    config xdc.runtime.Assert.Id A_invalidFrameQBufMgrId =
        {msg: "A_invalidFrameQBufMgrId: Invalid FrameQBufMgr id specified"};

/*!
     *  Assert raised when n invalid shared region address  specified.
     */
    config xdc.runtime.Assert.Id A_memoryAllocFail =
        {msg: "A_memoryAllocFail: Memory_alloc failed"} ;

   /*!
     *  Assert raised when null handle is returned from  modules.
     */
    config xdc.runtime.Assert.Id A_nullHandle =
        {msg: "A_nullHandle: Null Handle"};

    /*!
     *  Error raised when Invalid args passed.
     */
    config Error.Id E_InvalidArg  = {
        msg: "E_InvalidArg : Invalid Arguments passed"
    };

    /*!
     *  Assert raised when local protection provided is not sufficient for 
     * ClientNotifyMgr.
     */
    config xdc.runtime.Assert.Id A_localProtection  = {
        msg: "A_localProtection: Insufficient local protection "
    };

    /*!
     *  Error raised when Memory alloc failed.
     */
    config Error.Id E_Memory  = {
        msg: "E_Memory  : Memory alloc failed"
    };

    /*!
     *  Error raised when FrameQ is not created.
     */
    config Error.Id E_FrameQNotCreated  = {
        msg: "E_FrameQNotCreated: FrameQ instance is not created"
    };

    /*!
     *  Error raised when FrameQ is not Found in NameServer.
     */
    config Error.Id E_FrameQNotFound  = {
        msg: "E_FrameQNotFound: FrameQ instance is not found in NameServer"
    };

    /*!
     *  Error raised when FrameQ is already created.
     */
    config Error.Id E_FrameQCreated = {
        msg: "E_FrameQCreated: FrameQ instance is already created"
    };

    /*!
     *  Error raised when GateMP Create failed.
     */
    config Error.Id E_GateMPCreate  = {
        msg: "E_GateCreate: Gate instance Create Failed"
    };

    /*!
     *  Error raised when GateMP open failed.
     */
    config Error.Id E_GateMPOpen  = {
        msg: "E_GateMPOpen: Gate instance Open Failed"
    };

    /*!
     *  Error raised when ClientNotifyMgr Create failed.
     */
    config Error.Id E_ClientNotifyMgrCreate  = {
        msg: "E_ClientNotifyMgrCreate: ClientNotifyMgr instance Create Failed"
    };

    /*!
     *  Error raised when ClientNotifyMgrOpen failed.
     */
    config Error.Id E_ClientNotifyMgrOpen  = {
        msg: "E_ClientNotifyMgrOpen: ClientNotifyMgrOpen instance Open Failed"
    };
    /*!
     *  Error raised when FrameQBufMgr_open failed.
     */
    config Error.Id E_FrameQBufMgrOpen  = {
        msg: "E_FrameQBufMgrOpen: FrameQBufMgr instance Open Failed"
    };

    /*!
     *  Error raised when Insufficient shared memory provided to create call.
     */
    config Error.Id E_InvSize  = {
        msg: "E_InvSize: SharedMemory provided to create call is less than the instance shared memory requirements"
    };

    /*!
     *  Error raised when Insufficient shared addrprovided to create call.
     */
    config Error.Id E_InvAddr  = {
        msg: "E_InvAddr: Shared Addr provide to create call is NULL"
    };

    /*!
     *  Error raised when buffersize is not aligned to Cache line size.
     */
    config Error.Id E_CacheAlign  = {
        msg: "E_CacheAlign: BufferSize is not Cache Aligned"
    };

    /*!
     *  Error raised when postinitialize API failed 
     */
    config Error.Id E_Fail  = {
        msg: "E_Fail: Api failed"
    };

    /*!
     *  ======== OpenParams ========
     *  mouduleconfig structure.
     *
     *  @field(commonOpenParams)  Open params common to all the implementations.
     *                            see {@link CommonOpenParams}
     */
    struct OpenParams {
        CommonOpenParams commonOpenParams;
    };

    /*!
     * ================ open ========================
     *  API to get a open the  created FrameQ instance
     *
     *  The instance must be created before openinig it.
     * @param (handlePtr)  Handle to the created FrameQ instance will
     *                     be returned through this pointer.
     * @param(openParams)  Instance open params. Provide name to open the
     *                     instance.
     * @param(eb)          Error Block.
     *
     * @b(returns)         Returns  positive value if API is success.
     */
     Int32 open (OpenParams * openParams, Handle     *handlePtr );

    /*!
     * ================ openByAddr ========================
     *  API to get a open the  created FrameQ instance
     *
     *  The instance must be created before openinig it.
     * @param (handlePtr)  Handle to the created FrameQ instance will
     *                     be returned through this pointer.
     * @param(openParams)  Instance open params. provide sharedAddr to open the
     *                     instance.
     * @param(eb)          Error Block.
     *
     * @b(returns)         Retruns  positive value if API is success.
     */

       //Int32 openByAddr (Handle      *handlePtr,
         //              OpenParams  *openParams );
    /*!
     *  ======== close ========
     *  Function to close the dynamically opened instance using {@link #open} call.
     *
     *
     *  @b(returns)                 returns 0 if  successfully registered the call back function.
     *
     */
     Int32 close(Handle * handlePtr);

    /*!
     * =============== sharedMemReq =========================
     *  API to find out the instance level memory requirements.
     *
     * @param(params)    create params
     *
     * @b (returns)      Amount of shared memory required.
     *
     */
     UInt32 sharedMemReq (const Params *params) ;

     instance:

     config CreateParams commonCreateParams = {size:0,
                                               ctrlInterfaceType:INTERFACE_NONE,
                                               name:null,
                                               openFlag:false};

    /*!
     *  IGateProvider used for critical region management of the shared memory
     *
     */
    config GateMP.Handle gate = null;

    /*!
     *============localProtect ================
     * Default local protection level for the module instance.
     *
     */
    config  GateMP.LocalProtect localProtect = GateMP.LocalProtect_INTERRUPT;

    /*!
     *============remoteProtect ================
     * Default multiprocessor protection for the module instances.
     *
     */
    config  GateMP.RemoteProtect remoteProtect = GateMP.RemoteProtect_SYSTEM;

    /*!
     *  ======== sharedAddr ========
     *  Virtual Address of the shared memory.Must be mapped
     *  on other processors if instance needs to be accessible
     *  from other processors.
     *
     *  The creator must supply the shared memory that
     *  this will use to maintain shared state information.
     *  Required parameter if regionId is not specified.
     */
    config Ptr sharedAddr = null;

    /*!
     *  ======== sharedAddrSize ========
     *  Size of shareAddr
     *
     *  Can use the {@link #sharedMemReq} or {@link #sharedMemReqMeta} call to
     *  determine the required size.
     *
     *  Required parameter.
     */
    config UInt32 sharedAddrSize = 0;

    /*!
     *  Shared region ID
     *
     *  The ID corresponding to the shared region in which this shared instance
     *  is to be placed.
     */
    config UInt32 regionId = 0;

    /*!
     *  Max number of readers supported for this instance.
     *
     */
    config UInt32  numReaders = 1;

    /*!
     *  ======== numQueues ========
     *  Number of queues( Reader can retrieve frames from these queue by
     *  mentioning the queueId in getv calls.) supported for each reader.
     *
     */
    config UInt32  numQueues = 1;

    /*!
     * cpuAccessFlags if createMode is specified as  Reader or writer. Not used  if createMode is CREATE_MODE_NONE.
     */
    config UInt32 cpuAccessFlags = 0 ;

    /*!
     *  ======== frameQBufMgrName ========
     *  FrameQ bufMgr Name that needs to be used.
     */
    config String frameQBufMgrName = null ;

    /*!
     *  ======== frameQBufMgrSharedAddr ========
     *  Shared Address  of the FrameQBufMgr instance that needs to be used in
     *  FrameQ.Not required if {@link #frameQBufMgrName} is provided.
     */
     config Ptr frameQBufMgrSharedAddr = null ;

    /*! @_nodoc
     *  ======== setOpenFlags ========
     *  Function to set  the mode  for the  given handle. Internal function called by instance  open call .
     *
     *  @param(openParams)        open params to set . see {@link #OpenParams}.
     *
     *  @b(returns)       returns 0 if  successfully set the params for the given handle.
     *
     */
     Int32  setOpenFlags (OpenParams * openParams) ;

     internal:
  /*!
     *  ======== Module_State ========
     *  Module  state Object structure.
     *
     *  @field(nameServer)  Handle to the local NameServer used for storing FrameQ instance information.
     *                      Created by Front end FrameQ
     *
     */

    struct Module_State {
        NameServer.Handle     nameServer;
        UInt32                nameServerType;
        Config                cfg;
        UInt32                refCount;
        Ptr                   defaultInstGate;
    };

   /*!
     *  ======== Instance_State ========
     *  Instance  state Object structure.
     *
     *  @field(name)             Name of the instance.
     *  @field(nsKey)            Holds return value of Nameserver_add(for removing NS entry).
     *  @field(regionId)         region id of the shared region from where instance control
     *                           structure is allocated from.
     *  @field(allocSize)        Size of the memory allocated  for the instance
     *                           control structure from the shmIndex.
     *  @field(gate)             Gate for critical region management.
     *  @field(creGate)          Indicates whether gate is created internally.
     *  @field(gateMPAllocSize)  Indicates size of the allocated memory for the
     *                           instance GateMP if it is created internally.
     *  @field(instGateSharedAddr)Indicates sharedAddr of the allocated memory
     *                            for the instance GateMP if it is created
     *                            internally.Valid for creator only.
     *  @field(objType)          Denotes type of object.Dynamic create or static create or Dynamic open or static open.
     *  @field(objMode)          Denotes mode of the object.Reader  or writer or none.
     *  @field(minAlign)         Alignment of the controls structures. 
     *                           usually equal to cacheline size of the shared region.
     *  @field(pnameserverEntry) Pointer to the namserver info located the start
     *                           of the instance shared Addr.
     *  @field(controlStruct)    Pointer to the shared instance control structure.
     *  @field(virtClientHandle) Pointer to the Client's shared info. For reader this points to the {@link #ReaderClient}
     *                           For writer this points to the {@link #WriterClient}
     *  @field(frameQBufMgrHandle)  Handle of FrameBufMgr that this instance is using.
     *  @field(frameQBufMgrId)      Id of the FrameBufMgr that this instance is using.
     *  @field(attrs)           Pointer to attribute structure in shared memory.
     *  @field(writer)          Pointer to writer control info in shared memory.
     *  @field(reader)          Pointers to reader control info in shared memory.
     *  @field(fqueue)          Pointers to fqueue info {@link #FrameQueue}.
     *  @field(creCliGate)   To keep track of if clientnotifymgr gate is created internally .
     *
     */
    /* not for client use */
     struct Instance_State {
            String                     name;
            Ptr                        nsKey;
            UInt32                     regionId;       /* region id of the shared region  */
            SizeT                      allocSize;      /* Shared memory allocated       */
            GateMP.Handle              gate;
            Bool                       creGate;
            UInt32                     gateMPAllocSize;
            Ptr                        instGateSharedAddr;
            UInt32                     objType;
            UInt32                     ctrlInterfaceType;            
            UInt32                     objMode;
            UInt32                     minAlign;
            Ptr                        pnameserverEntry;
            volatile Attrs            *attrs;
            volatile WriterClient     *writer;
            volatile ReaderClient     **reader;
            volatile FrameQueue       **frmQueue;
            volatile Ptr               virtClientHandle ;
            Ptr                        clientNotifyMgrHandle ;
            GateMP.Handle              clientNotifyMgrGate;
            FrameQBufMgr.Handle        frameQBufMgrHandle ;
            UInt32                     frameQBufMgrId;
            UInt32                     notifyRegId ;
            Bool                       ctrlStructCacheFlag ;
            Bool                       frmHdrBufCacheFlag ;
            Bool                       frmBufCacheFlag[8] ;
            Bool                       bufCpuAccessFlag[8];
            ti.sdo.ipc.ListMP.Object ***frameList;  /* Lists of frames */
            Bool                      creCliGate;
     };
}
